// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <ClanLib/clanlayer2.h>

#include "mouse.hh"
#include "pingu.hh"
#include "button_panel.hh"
#include "actions/pingu_actions.hh"

CL_ResourceManager* ButtonPanel::resource = 0;
CL_Surface* ButtonPanel::button_cap;
CL_Font*    ButtonPanel::font;

ButtonPanel::ButtonPanel()
{
  need_redraw = true;
  pressed_button = 2;
  button.resize(8);
  
  button[0] = new Button(  0, 424, "Buttons/exploder", resource);
  button[1] = new Button( 38, 424, "Buttons/walker", resource);
  button[2] = new Button( 76, 424, "Buttons/bridger", resource);
  button[3] = new Button(114, 424, "Buttons/blocker", resource);
  button[4] = new Button(152, 424, "Buttons/floater", resource);
  button[5] = new Button(190, 424, "Buttons/empty", resource);
  button[6] = new Button(228, 424, "Buttons/empty", resource);
  button[7] = new Button(266, 424, "Buttons/empty", resource);
}

ButtonPanel::~ButtonPanel()
{
}

void
ButtonPanel::let_move() 
{
  for(int i=0; i < button.size(); ++i) {
    button[i]->let_move();
  }

  if (Mouse::get_y() > 424 && Mouse::is_pressed(0)) {
    for(int i=0; i < 8; ++i) {
      if (Mouse::get_x() >= i * 38 && Mouse::get_x() < ((i + 1) * 38)) {
	pressed_button = i;
	break;
      }
    }
  }
}

PinguAction*
ButtonPanel::get_action()
{
  switch (pressed_button) {
  case 4:
    return new Floater;
  case 0:
    return new Bomber;
  case 1:
    //    return walker;
  case 2:
    //    return bridger;
  case 3:
    return new Blocker;
  default:
    cout << "ButtonPanel: Button not defined" << endl;
    return 0;
  }
}

void 
ButtonPanel::draw() 
{
  need_redraw = true;
  if (need_redraw) {
    // draw the buttons
    for(int i=0; i < button.size(); ++i) {
      button[i]->draw();
    }
    need_redraw = false;
  }

  // draw the capture rectangle for the active button
  button_cap->put_screen(pressed_button*38, 424);
}

void
ButtonPanel::load_data(CL_ResourceManager* res)
{
  resource = res;
  button_cap      = CL_Res_Surface::load("Buttons/button_cap", *resource);
  font            = CL_Res_Font::load("Fonts/font", *resource);
}

/* EOF */
