// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <iostream>
#include <cstdio>
#include <string>
#include <fstream>

#include "plf.hh"

PLF::PLF()
{
  x_pos = y_pos = 0;
  pingus = 50;
}


PLF::PLF(const string& filename)
{
  char token[256];
  char* dir;

  maptype = UNDEF;
  x_pos = y_pos = 0;

  ifstream in(filename.c_str());

  if (!in) {
    perror(filename.c_str());
    exit(EXIT_FAILURE);
  }

  // Get directory
  dir = strdup(filename.c_str());
  for(int i=strlen(dir); i>=0; --i) {
    if (dir[i] == '/') {
      dir[i+1] = '\0';
      break;
    }
  }

  bg  = new char[1024];
  fg  = new char[1024];
  col = new char[1024];
  char tmp[1024];

  //  strcpy(bg, dir);
  strcpy(fg, dir);
  strcpy(col, dir);

  hotspot_t spot;
  exit_data exit_p;
  entrance_data entrance_p;
  trap_data trap;

  while (!in.eof()) {
    in >> token;

    switch (get_token(token)) {
    case BG:
      in >> bg;
      break;
    case FG:
      in >> tmp;
      strcat(fg, tmp);
      break;
    case COL:
      in >> tmp;
      strcat(col, tmp);
      cout << "PLF:" << col << endl; 
      break;
    case EMPTY:
      break;
    case COMMENT:
      in.ignore(255, '\n');
      break;
    case START:
      in >> x_pos;
      in >> y_pos;
      break;
    case HOTSPOT_BG:
      in >> spot.name;
      in >> spot.x_pos;
      in >> spot.y_pos;
      hotspot_bg.push_back(spot);
      break;
    case HOTSPOT_FG:
      in >> spot.name;
      in >> spot.x_pos;
      in >> spot.y_pos;
      hotspot_fg.push_back(spot);
      break;
    case EXIT:
      in >> exit_p.name;
      in >> exit_p.x_pos;
      in >> exit_p.y_pos;
      exit_v.push_back(exit_p);
      break;
    case ENTRANCE:
      in >> entrance_p.name;
      in >> entrance_p.x_pos;
      in >> entrance_p.y_pos;
      in >> entrance_p.release_rate;
      entrance_v.push_back(entrance_p);
      break;
    case MAPTYPE:
      in >> token;
      if (strcmp(token, "BMP") == 0) {
	maptype = BMP;
      } else if (strcmp(token, "TILE") == 0) {
	maptype = TILE;
      } else {
	cout << "Syntax Error in PLF file! Wrong maptyp!" << endl;
	exit(EXIT_FAILURE);
      }
      break;
    case TRAP:
      in >> trap.name;
      in >> trap.x_pos;
      in >> trap.y_pos;
      in >> trap.data;
      traps.push_back(trap);
      break;
    case PINGUS:
      in >> pingus;
      break;
    case UNKNOWN:
      cout << "Syntax Error in PLF file!" << endl;
      exit(EXIT_FAILURE);
      break;
    }
  }
  
  if (maptype == UNDEF) {
    cout << "No maptype set in PLF-file" << endl;
    exit(EXIT_FAILURE);
  }
}

plf_token
PLF::get_token(char* token)
{
  toupper(token);

  if (strcmp(token, "BG:") == 0) {
    return BG;
  } else if (strcmp(token, "FG:") == 0) {
    return FG;
  } else if (strcmp(token, "COL:") == 0) {
    return COL;
  } else if (strcmp(token, "MAPTYPE:") == 0) {
    return MAPTYPE;
  } else if (strcmp(token, "") == 0) {
    return EMPTY;
  } else if (strcmp(token, "//") == 0) {
    return COMMENT;
  } else if (strcmp(token, "START:") == 0) {
    return START;
  } else if (strcmp(token, "HOTSPOT_FG:") == 0) {
    return HOTSPOT_FG;
  } else if (strcmp(token, "HOTSPOT_BG:") == 0) {
    return HOTSPOT_BG;
  } else if (strcmp(token, "EXIT:") == 0) {
    return EXIT;
  } else if (strcmp(token, "ENTRANCE:") == 0) {
    return ENTRANCE;
  } else if (strcmp(token, "TRAP:") == 0) {
    return TRAP;
  } else if (strcmp(token, "PINGUS:") == 0) {
    return PINGUS;
  }
  cout << "Current Token: " << token << endl;
  return UNKNOWN;
}

const char*
PLF::get_bg()
{
  return bg;
}

const char*
PLF::get_fg()
{
  return fg;
}

const char*
PLF::get_pngfile()
{
  //  return (filename + ".png").c_str();  
}

MapType
PLF::map_type()
{
  return maptype;
}

char const*
PLF::get_mapfile()
{
  return col;
}

int
PLF::get_startx()
{
  return -x_pos;
}

int
PLF::get_starty()
{
  return y_pos;
}

vector<hotspot_t>
PLF::get_hotspot_fg()
{
  return hotspot_fg;
}

vector<hotspot_t>
PLF::get_hotspot_bg()
{
  return hotspot_bg;
}

vector<exit_data>
PLF::get_exit()
{
  return exit_v;
}

vector<entrance_data>
PLF::get_entrance()
{
  return entrance_v;
}

vector<trap_data> 
PLF::get_traps()
{
  return traps;
}

int
PLF::get_pingus()
{
  return pingus;
}

/* EOF */
